/* -------------------- spi_uart.h -----------------

     stellt einfacheFunktionen fuer SPI und UART
     Datentransfer zur Verfuegung.

     15.04.2016   R. Seelig
  -------------------------------------------------- */


#ifndef in_spiuart
  #define in_spiuart

  #include <avr/io.h>
  #include <avr/pgmspace.h>
  #include <util/delay.h>

/* --------------------------------------------------
    Registernamen der seriellen Schnittstelle von
    ATmegaxx8 fuer andere Controller verfuegbar
    machen
   -------------------------------------------------- */

  #if defined (__AVR_ATmega8__) || defined (__AVR_ATtiny2313__)  || defined (__AVR_ATmega8515__)

    #define UBRR0       UBRR
    #define UBRR0L      UBRRL
    #define UBRR0H      UBRRH
    #define UCSR0A      UCSRA
    #define UCSR0B      UCSRB
    #define UCSR0C      UCSRC
    #define U2X0        U2X
    #define RXEN0       RXEN
    #define TXEN0       TXEN
    #define UCSZ00      UCSZ0
    #define UCSZ01      UCSZ1
    #define UDRE0       UDRE
    #define UDR0        UDR
    #define RXC0        RXC

  #endif

  /* --------------------------------------------------
       Zurordnung der Pinbelegung der SPI Schnitt-
       stelle
     -------------------------------------------------- */

  #define SPI_DDR         DDRB
  #define SPI_PORT        PORTB
  #define SPI_INPUT       PINB
  #define SPI_MOSI_PIN    PB3             // Arduino digital 11
  #define SPI_MISO_PIN    PB4             //      "          12
  #define SPI_CLK_PIN     PB5             //      "          13
  #define SPI_SS_PIN      PB2             //      "          10
  #define SPI_RST_PIN     PB2             //      "          10


  /* --------------------------------------------------
        grundsaetzliche Funktionen und Macros der
        seriellen  Schnittstelle
     -------------------------------------------------- */

  void     uart_init(uint32_t baud);
  void     uart_putchar(uint8_t ch);
  uint8_t  uart_getchar(void);
  uint8_t  uart_ischar(void);
  void     uart_putramstring(uint8_t *p);
  void     uart_putromstring(const uint8_t *dataPtr);
  #define  uart_prints(tx)   uart_putromstring(PSTR(tx))         // Benutzung: uart_prints("Hallo Welt\n\r");
  void     printhex(uint8_t value);
  uint8_t  gethex(char echo);
  void     printhex16(uint16_t value);
  uint16_t  gethex16(char echo);
  void     readstr(char *string, char anz);
  void     crlf(void);


  /* --------------------------------------------------
        grundsaetzliche Funktionen und Macros des
        SPI - Interfaces
     -------------------------------------------------- */

  void     spi_init(uint8_t speed, uint8_t clkx2);
  uint8_t  spi_out(uint8_t data);
  #define  spi_read()      ( spi_out(0xff) )
  void     spi_setrst(void);
  void     spi_clrrst(void);
  #define  spi_setmosi()   ( SPI_PORT |= (1 << SPI_MOSI_PIN) )
  #define  spi_clrmosi()   ( SPI_PORT &= ~(1 << SPI_MOSI_PIN) )
  #define  spi_setclk()    ( SPI_PORT |= (1 << SPI_CLK_PIN) )
  #define  spi_clrclk()    ( SPI_PORT &= ~(1 << SPI_CLK_PIN) )
  #define  spi_rdmiso()    ( (SPI_INPUT & (1 << SPI_MISO_PIN)) )
  #define  spi_enable()    ( SPCR |= (1 << SPE) )
  #define  spi_disable()   ( SPCR &= ~(1 << SPE) )


  extern char rinv;
  extern char clkinv;
  extern char aktfreq;
#endif
